@@ -1,11 +1,93 @@
+##' @title addRasterLegend
+##'
+##' @description Adds a legend to an existing raster plot, with some additional manual control
 
-addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2, shortFrac = 0.02, longFrac = 0.3, axisOffset = 0, border = TRUE, ramp = "terrain", isInteger = 'auto', ncolors = 64, breaks = NULL, minmax = NULL, locs = NULL, cex.axis = 0.8, labelDist = 0.7, digits = 2, ...) {
+##' @param r the rasterLayer object that has been plotted
+##' @param direction direction of color ramp. If omitted, then direction is automatically 
+##'	 inferred, otherwise can be specified as \code{horizontal} or \code{vertical}.
+##' @param side side for tick marks, see \code{\link{axis}} documentation. 
+##'		Automatically inferred if omitted.
+##' @param location either a location name (see \code{Details}), or coordinates for 
+##'		the corners of the bar legend \code{c(xmin, xmax, ymin, ymax)}.
+##' @param nTicks number of tick marks, besides min and max.
+##' @param adj if location is top, left, bottom or right, use this argument to adjust the 
+##'		location of the legend, defined in percent of the figure width. See Details 
+##'		for additional information. 
+##' @param shortFrac Percent of the plot width range that will be used as the short 
+##' 	dimention of the legend. Only applies to preset location options.
+##' @param longFrac Percent of the plot width range that will be used as the long dimention 
+##'		of the legend. Only applies to preset location options.	
+##' @param axisOffset distance from color bar for labels, as a percent of the plot width.
+##' @param border logical, should the color legend have a black border
+##' @param ramp either a vector of color names for defining the color ramp, 
+##'		or "terrain" (default raster behavior)
+##' @param isInteger If \code{auto}, automatically determines if raster is made up of integer 
+##'		values, otherwise \code{TRUE} or \code{FALSE}
+##' @param ncolors grain size of color ramp
+##' @param breaks If a custom set of color breaks were used in plotting the raster, pass 
+##'		those color breaks here. This overrides the minmax option. 
+##' @param minmax min and max values from which the color ramp will be derived. If left
+##' 	as \code{NULL}, the min and max of the raster will be used.
+##' @param locs locations of tick marks, if \code{NULL} automatically placed
+##' @param cex.axis size of axis labels
+##' @param labelDist distance from axis to axis labels (passed to \code{mgp})
+##' @param digits number of decimal places for labels
+##' @param bigmark character used to separate thousands and millions, passed 
+##' 	to \code{\link{format}}
+##' @param ... additional parameters to be passed to \code{\link{axis}}.
+
+##' @details
+##'		A number of predefined locations exist in this function to make it easy to 
+##'		add a legend to a raster plot. 
+##'		Preset \code{locations} are: \code{topleft}, \code{topright}, \code{bottomleft}, 
+##'		\code{bottomright}, \code{left}, \code{right}, \code{top} and \code{bottom}. 
+##'		If more fine-tuned control is desired, then a numeric vector of length 4 can be 
+##'		supplied to \code{location}, specifying the min x, max x, min y and max y 
+##'		values for the legend. 
+##'		Additionally, the \code{adj} argument can be used to more intuitively adjust where
+##' 	the legend is placed. \code{adj} is defined as a percentage of the figure width or 
+##' 	height, left to right, or bottom to top, respectively. For example, if the legend is at 
+##' 	the bottom, \code{adj = 0.8} will place the legend 80% of the distance from the top of 
+##' 	the figure, horizontally centered.  
+##'		See examples.
+
+##' @return Invisibly returns a list with the following components.
+##' \itemize{
+##'	\item{coords}{2-column matrix of xy coordinates for each color bin in the legend.}
+##'	\item{width}{Coordinates for the short dimension of the legend.}
+##'	\item{pal}{the color ramp}
+##'	\item{tickLocs}{the tick mark locations in plotting units}
+##'	\item{labels}{the values associated with those tick locations.}
+##' }
+
+##' @author Pascal Title
+
+##' @examples
+##' library(raster)
+##' r <- raster(system.file("external/test.grd", package="raster"))
+##' 
+##' plot(r, legend = FALSE)
+##' addRasterLegend(r, location = 'right')
+##' addRasterLegend(r, location = 'top')
+##' 
+##' # fine-tune placement
+##' plot(r, legend = FALSE)
+##' addRasterLegend(r, location=c(181000, 181100, 330500, 331500), side = 4)
+ 
+##' @export
+
+
+addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2, adj = NULL, shortFrac = 0.02, longFrac = 0.3, axisOffset = 0, border = TRUE, ramp = "terrain", isInteger = 'auto', ncolors = 64, breaks = NULL, minmax = NULL, locs = NULL, cex.axis = 0.8, labelDist = 0.7, digits = 2, bigmark = '', ...) {
 		
+	if (inherits(r, 'speciesRaster')) {
+		r <- r[[1]]
+	}
+	
 	if (!inherits(r, 'RasterLayer')) {
-		stop("r must be a RasterLayer.");
+		stop("r must be a speciesRaster object or RasterLayer.");
 	}
 		
-	if(!hasArg('direction')) {
+	if(!methods::hasArg('direction')) {
 		direction <- 'auto'
 	}
 	
@@ -23,19 +105,21 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 		stop('isInteger must be "auto", TRUE or FALSE.')
 	}
 	
-	if (length(ramp) == 1) {
-		if (ramp == 'terrain') {
-			pal <- rev(terrain.colors(ncolors));
+	if(!methods::hasArg('ramp')) {
+		pal <- rev(grDevices::terrain.colors(ncolors));
+	} else {
+		if (inherits(ramp, 'function')) {
+			pal <- ramp(ncolors)
+		} else {
+			pal <- grDevices::colorRampPalette(ramp)(ncolors)
 		}
-	} else if (length(ramp) > 1) {
-		pal <- colorRampPalette(ramp)(ncolors);
 	}
 	
 	#if minmax provided, use to generate linear color breaks
 	if (is.null(minmax)) {
-		colorbreaks <- seq(minValue(r), maxValue(r), length.out = (ncolors+1));
+		colorbreaks <- seq(raster::minValue(r), raster::maxValue(r), length.out = (ncolors+1));
 	} else {
-		colorbreaks <- seq(minmax[1], minmax[2], length.out = (ncolors+1));
+		colorbreaks <- seq(minmax[1], minmax[2], length.out = (ncolors + 1));
 	}
 	
 	#if supplied, use custom set of breaks
@@ -60,6 +144,8 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 	maxY <- maxY - yrange * 0.05
 	
 	if (is.character(location)) {
+		
+		locChar <- location
 	
 		if (location == 'topleft' & direction %in% c('auto', 'vertical')) {
 			location <- vector('numeric', length = 4);
@@ -161,6 +247,23 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 			direction <- 'horizontal'
 		}
 	}
+	
+	# if adj argument provided, use to adjust location
+	# currently only implemented for locations bottom, left, top, right
+	if (!is.null(adj)) {
+		if (adj > 1) {
+			stop('adj must be supplied as a percentage.')
+		}
+		if (locChar %in% c('top', 'bottom')) {
+			Ydiff <- diff(location[3:4])
+			location[3] <- minY + (maxY - minY) * adj
+			location[4] <- location[3] + Ydiff
+		} else if (locChar %in% c('left', 'right')) {
+			Xdiff <- diff(location[1:2])
+			location[1] <- minX + (maxX - minX) * adj
+			location[2] <- location[1] + Xdiff			
+		}
+	}
 
 	# infer direction based on dimensions of legend box
 	if (direction == 'auto') {
@@ -178,7 +281,7 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 	}
 	
 	#determine side for labels based on location in plot and direction
-	if (!hasArg('side')) {
+	if (!methods::hasArg('side')) {
 		if (direction == 'vertical') { #side = 1 or 4
 			if (mean(location[1:2]) <= mean(par('usr')[1:2])) {
 				side <- 4;
@@ -219,8 +322,8 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 		# if max value is included, add in manually
 		tol <- 1e-10
 		maxValueIncluded <- FALSE
-		if (any(abs(maxValue(r) - locs) < tol)) {
-			locs <- locs[which(abs(maxValue(r) - locs) >= tol)]
+		if (any(abs(raster::maxValue(r) - locs) < tol)) {
+			locs <- locs[which(abs(raster::maxValue(r) - locs) >= tol)]
 			maxValueIncluded <- TRUE
 		}
 		tickLocs <- x[sapply(locs, function(x) which((x >= z[,1] & x < z[,2]) == TRUE)),1];
@@ -239,7 +342,7 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 	
 	#if raster values are integer, then make legend have integer values
 	if (isInteger == 'auto') {
-		randomSample <- sample(values(r)[which(!is.na(values(r)))], size=1000, replace = TRUE)
+		randomSample <- sample(raster::values(r)[which(!is.na(raster::values(r)))], size=1000, replace = TRUE)
 		if (identical(randomSample, trunc(randomSample))) {
 			tx <- round(tx, 0)
 		}
@@ -258,20 +361,28 @@ addRasterLegend <- function(r, direction, side, location = 'right', nTicks = 2,
 		rect(location[1], location[3], location[2], location[4], border='black', xpd = NA);
 	}
 	
+	if (bigmark != '') {
+		tickLabels <- format(tx, big.mark = bigmark)
+	} else {	
+		tickLabels <- round(tx, digits)
+	}
+	
 	#add tickmarks
 	if (side == 1) { #bottom
-		axis(side, at = tickLocs, pos = location[3] - axisOffset, labels = round(tx, digits), xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
+		axis(side, at = tickLocs, pos = location[3] - axisOffset, labels = tickLabels, xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
 	} 
 	if (side == 3) { #top
-		axis(side, at = tickLocs, pos = location[4] + axisOffset, labels = round(tx, digits), xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
+		axis(side, at = tickLocs, pos = location[4] + axisOffset, labels = tickLabels, xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
 	}
 	if (side == 2) { #left
-		axis(side, at = tickLocs, pos = location[1] - axisOffset, labels = round(tx, digits), xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
+		axis(side, at = tickLocs, pos = location[1] - axisOffset, labels = tickLabels, xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
 	}
 	if (side == 4) { #right
-		axis(side, at = tickLocs, pos = location[2] + axisOffset, labels = round(tx, digits), xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
+		axis(side, at = tickLocs, pos = location[2] + axisOffset, labels = tickLabels, xpd = NA, las = 1, cex.axis = cex.axis, mgp = c(3, labelDist, 0), ...);
 	}
 }
 	
 	
 
+	
+
